﻿using System.IO;
using System.Text;

namespace Cs2dMapLib.IO
{
    /// <summary>
    /// It used to map reading
    /// </summary>
    class MapReader
    {
        /// <summary>
        /// File stream
        /// </summary>
        private FileStream stream;

        /// <summary>
        /// Binary reader
        /// </summary>
        private BinaryReader reader;


        /// <summary>
        /// Opens the map reading along a predetermined path
        /// </summary>
        /// <param name="path">Path to read file</param>
        public MapReader(string path)
        {
            stream = new FileStream(path, FileMode.Open);
            reader = new BinaryReader(stream);
        }


        /// <summary>
        /// Read string
        /// </summary>
        public string ReadString()
        {
            string str = "";
            byte b;
            while ((b = reader.ReadByte()) != 10)
                if (b != 13)
                    str += (char)b;
            return str;
        }

        /// <summary>
        /// Read integer
        /// </summary>
        public int ReadInt()
        {
            return reader.ReadInt32();
        }

        /// <summary>
        /// Read byte
        /// </summary>
        public byte ReadByte()
        {
            return reader.ReadByte();
        }

        /// <summary>
        /// Read string array
        /// </summary>
        /// <param name="Count">The length of the returned array</param>
        public string[] ReadStrings(int Count)
        {
            string[] str = new string[Count];
            for (int i = 0; i < Count; i++)
                str[i] = ReadString();
            return str;
        }

        /// <summary>
        /// Read integer array
        /// </summary>
        /// <param name="Count">The length of the returned array</param>
        public int[] ReadInts(int Count)
        {
            int[] ints = new int[Count];
            for (int i = 0; i < Count; i++)
                ints[i] = reader.ReadInt32();
            return ints;
        }

        /// <summary>
        /// Read byte array
        /// </summary>
        /// <param name="Count">The length of the returned array</param>
        public byte[] ReadBytes(int Count)
        {
            return reader.ReadBytes(Count);
        }

        /// <summary>
        /// Ends the reading and closes the file
        /// </summary>
        public void Close()
        {
            reader.Close();
            stream.Close();           
        }
    }

    /// <summary>
    /// Used to create and write to the data from the map file
    /// </summary>
    class MapWriter
    {
        /// <summary>
        /// File stream
        /// </summary>
        private FileStream stream;

        /// <summary>
        /// Binary writer
        /// </summary>
        private BinaryWriter writer;


        /// <summary>
        /// It creates a file for writing at a given path
        /// </summary>
        /// <param name="path">Path to save file</param>
        public MapWriter(string path)
        {
            stream = new FileStream(path, FileMode.Create);
            writer = new BinaryWriter(stream);
        }


        /// <summary>
        /// Write string
        /// </summary>
        /// <param name="String">The object to be written</param>
        public void Write(string String)
        {
            writer.Write(Encoding.UTF8.GetBytes(String + "\r\n"));
        }

        /// <summary>
        /// Write integer
        /// </summary>
        /// <param name="Int">The object to be written</param>
        public void Write(int Int)
        {
            writer.Write(Int);
        }

        /// <summary>
        /// Write byte
        /// </summary>
        /// <param name="Byte">The object to be written</param>
        public void Write(byte Byte)
        {
            writer.Write(Byte);
        }

        /// <summary>
        /// Write string array
        /// </summary>
        /// <param name="Strings">The objects array to be written</param>
        public void Write(string[] Strings)
        {
            foreach (string str in Strings)
                Write(str);               
        }

        /// <summary>
        /// Write integer array
        /// </summary>
        /// <param name="Ints">The objects array to be written</param>
        public void Write(int[] Ints)
        {
            foreach (int i in Ints)
                writer.Write(i);
        }

        /// <summary>
        /// Write byte array
        /// </summary>
        /// <param name="Bytes">The objects array to be written</param>
        public void Write(byte[] Bytes)
        {
            writer.Write(Bytes);
        }

        /// <summary>
        /// Ends the writing and closes the file
        /// </summary>
        public void Close()
        {
            writer.Close();
            stream.Close();
        }
    }
}
